==> cri-o_1.26.4~1.tar.gz: BUILD/internal/runtimehandlerhooks/default_cpu_load_balance_hooks.go <==
package runtimehandlerhooks

import (
	"context"
	"fmt"

	"github.com/cri-o/cri-o/internal/config/node"
	"github.com/cri-o/cri-o/internal/lib/sandbox"
	"github.com/cri-o/cri-o/internal/oci"
	"github.com/opencontainers/runc/libcontainer/cgroups"
)

// DefaultCPULoadBalanceHooks is used to run additional hooks that will configure containers for CPU load balancing.
// Specifically, it will define a PostStop that disables `cpuset.sched_load_balance` for a recently stopped container.
// This must be done because guaranteed pods with exclusive cpu access may be created after other containers are terminated,
// but before their cgroup is cleaned up. In this case, cpumanager will not load balancing the exclusive CPUs away from those pods,
// thus causing their `cpuset.sched_load_balance=1` to prevent the kernel from disabling load balancing.
// This is the only case it seeks to fix, and thus does not define any other members of the RuntimeHandlerHooks functions.
type DefaultCPULoadBalanceHooks struct{}

// No-op
func (*DefaultCPULoadBalanceHooks) PreStart(context.Context, *oci.Container, *sandbox.Sandbox) error {
	return nil
}

// No-op
func (*DefaultCPULoadBalanceHooks) PreStop(context.Context, *oci.Container, *sandbox.Sandbox) error {
	return nil
}

func (*DefaultCPULoadBalanceHooks) PostStop(ctx context.Context, c *oci.Container, s *sandbox.Sandbox) error {
	// Disable cpuset.sched_load_balance for all stale cgroups.
	// This way, cpumanager can ignore stopped containers, but the running ones will still have exclusive access.
	if node.CgroupIsV2() || c.Spoofed() {
		return nil
	}
	containerCgroup, containerCgroupParent, systemd, err := containerCgroupAndParent(s.CgroupParent(), c)
	if err != nil {
		return err
	}
	mgr, err := libctrManager(containerCgroup, containerCgroupParent, systemd)
	if err != nil {
		return err
	}
	path := mgr.Path("cpuset")
	if path == "" {
		return fmt.Errorf("failed to find cpuset for newly created cgroup")
	}
	return cgroups.WriteFile(path, "cpuset.sched_load_balance", "0")
}

==> cri-o_1.27.4~0.tar.gz: BUILD/internal/runtimehandlerhooks/default_cpu_load_balance_hooks.go <==
package runtimehandlerhooks

import (
	"context"

	"github.com/cri-o/cri-o/internal/lib/sandbox"
	"github.com/cri-o/cri-o/internal/oci"
)

// DefaultCPULoadBalanceHooks is used to run additional hooks that will configure containers for CPU load balancing.
// Specifically, it will define a PostStop that disables `cpuset.sched_load_balance` for a recently stopped container.
// This must be done because guaranteed pods with exclusive cpu access may be created after other containers are terminated,
// but before their cgroup is cleaned up. In this case, cpumanager will not load balancing the exclusive CPUs away from those pods,
// thus causing their `cpuset.sched_load_balance=1` to prevent the kernel from disabling load balancing.
// This is the only case it seeks to fix, and thus does not define any other members of the RuntimeHandlerHooks functions.
type DefaultCPULoadBalanceHooks struct{}

// No-op
func (*DefaultCPULoadBalanceHooks) PreStart(context.Context, *oci.Container, *sandbox.Sandbox) error {
	return nil
}

// No-op
func (*DefaultCPULoadBalanceHooks) PreStop(context.Context, *oci.Container, *sandbox.Sandbox) error {
	return nil
}

func (*DefaultCPULoadBalanceHooks) PostStop(ctx context.Context, c *oci.Container, s *sandbox.Sandbox) error {
	// Disable cpuset.sched_load_balance for all stale cgroups.
	// This way, cpumanager can ignore stopped containers, but the running ones will still have exclusive access.
	if c.Spoofed() {
		return nil
	}

	_, containerManagers, err := libctrManagersForPodAndContainerCgroup(c, s.CgroupParent())
	if err != nil {
		return err
	}

	return disableCPULoadBalancing(containerManagers)
}
